@@ -267,6 +267,12 @@ db_set_params (TrackerDBInterface *iface,
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
 
+	/* if we add direct access library (or enable running queries in parallel with updates,
+	   we need to disable read_uncommitted again
+	   however, when read_uncommitted is disabled, we need the custom page cache (wrapper)
+	   to avoid locking issues between long running batch transactions and queries */
+	tracker_db_interface_execute_query (iface, NULL, "PRAGMA read_uncommitted = True;");
+
 	if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
 		g_message ("  Setting page size to %d", page_size);
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA page_size = %d", page_size);
@@ -633,9 +639,20 @@ static void
 free_thread_interface (gpointer data)
 {
 	TrackerDBInterface *interface = data;
+	GHashTableIter iter;
+	gpointer value;
 
 	g_static_mutex_lock (&thread_ifaces_mutex);
-	g_hash_table_remove (thread_ifaces, g_thread_self ());
+
+	g_hash_table_iter_init (&iter, thread_ifaces);
+
+	while (g_hash_table_iter_next (&iter, NULL, &value)) {
+		if (value == data) {
+			g_hash_table_iter_remove (&iter);
+			break;
+		}
+	}
+
 	g_static_mutex_unlock (&thread_ifaces_mutex);
 
 	g_object_unref (interface);
@@ -889,8 +906,12 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	in_use_file = g_open (in_use_filename,
 	                      O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
 	                      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
-	fsync (in_use_file);
-	close (in_use_file);
+
+        if (in_use_file >= 0) {
+                fsync (in_use_file);
+                close (in_use_file);
+        }
+
 	g_free (in_use_filename);
 
 	tracker_db_manager_ensure_locale ();
