@@ -27,7 +27,7 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-#include <libtracker-client/tracker-client.h>
+#include <libtracker-client/tracker.h>
 #include <libtracker-common/tracker-common.h>
 
 #define ABOUT	  \
