@@ -58,6 +58,7 @@ const TestInfo tests[] = {
 	{ "anon/query", "anon/data", FALSE },
 	{ "ask/ask-1", "ask/data", FALSE },
 	{ "basic/base-prefix-3", "basic/data-1", FALSE },
+	{ "basic/compare-cast", "basic/data-1", FALSE },
 	{ "basic/predicate-variable", "basic/data-1", FALSE },
 	{ "basic/predicate-variable-2", "basic/data-1", FALSE },
 	{ "basic/predicate-variable-3", "basic/data-1", FALSE },
@@ -82,6 +83,8 @@ const TestInfo tests[] = {
 	{ "functions/functions-xpath-2", "functions/data-1", FALSE },
 	{ "functions/functions-xpath-3", "functions/data-1", FALSE },
 	{ "functions/functions-xpath-4", "functions/data-1", FALSE },
+	{ "functions/functions-xpath-5", "functions/data-1", FALSE },
+	{ "functions/functions-xpath-6", "functions/data-1", FALSE },
 	{ "graph/graph-1", "graph/data-1", FALSE },
 	{ "graph/graph-2", "graph/data-2", FALSE },
 	{ "graph/graph-3", "graph/data-3", FALSE },
@@ -95,6 +98,7 @@ const TestInfo tests[] = {
 	{ "sort/query-sort-5", "sort/data-sort-4", FALSE },
 	{ "subqueries/subqueries-1", "subqueries/data-1", FALSE },
 	{ "subqueries/subqueries-union-1", "subqueries/data-1", FALSE },
+	{ "subqueries/subqueries-union-2", "subqueries/data-1", FALSE },
 	/* Bracket error after WHERE */
 	{ "error/query-error-1", "error/query-error-1", TRUE, FALSE },
 	/* Unknown property */
